
from joints import *
from animatedfigure import *









class HierarchyParser(object):
    def __init__(self):
        pass
    
    
    def parse(self, lines):
        self.lines = lines
        self.current_line = 0
        
        
        root_joint = self.parse_joint()
        
        return root_joint
    
    
    def parse_joint(self):
        
        tokens = self.get_tokens_for_next_non_blank_line()
            
        joint_type = tokens[0]
        
        if joint_type == "End":
            joint_name = tokens[2]
        else:
            joint_name = tokens[1]
            
        
        
        self.next_line()
        tokens = self.get_tokens_for_next_non_blank_line()
        
        if tokens[0] != '{':
            raise ValueError, "Brace trouble"
        
        
        self.next_line()
        tokens = self.get_tokens_for_next_non_blank_line()
        
        offset = [float(tokens[1]), float(tokens[2]), float(tokens[3])]
        
        
        
        self.next_line()
        
        
        #create the new joint
        if joint_type == "ROOT":
            new_joint = RootJoint(joint_name, offset)
            
            tokens = self.get_tokens_for_next_non_blank_line()
            
            if tokens[0] != 'CHANNELS' and tokens[1] != '6':
                raise ValueError, "Channel parsing problem"
            
            self.next_line()
            
        elif joint_type == "JOINT":
            new_joint = InternalJoint(joint_name, offset)
            
            tokens = self.get_tokens_for_next_non_blank_line()
            
            if tokens[0] != 'CHANNELS' and tokens[1] != '3':
                raise ValueError, "Channel parsing problem"
            
            self.next_line()
            
        elif joint_type == "End":
            new_joint = LeafJoint(joint_name, offset)
            
        else:
            raise ValueError, "Bad Joint Type"
            
        
        tokens = self.get_tokens_for_next_non_blank_line()
        
        while tokens[0] != '}':
            
            
            
            child = self.parse_joint()
        
            new_joint.add_child(child)
            tokens = self.get_tokens_for_next_non_blank_line()
        
        
        self.next_line()
        
        
        return new_joint
            
            
            
            
            
            
            
            
    def next_line(self):
        self.current_line += 1
            
            
    def get_tokens_for_next_non_blank_line(self):
            
        while self.current_line < len(self.lines):
            
            tokens = self.lines[self.current_line].split()
            
            if len(tokens) > 0: 
                    return tokens
                
            self.current_line += 1
            
        raise ValueError, "Premature end of hierarchy"
        
    
    
    


class MotionParser(object):
    def __init__(self):
        pass
    
    
    def parse(self, lines):
        self.lines = lines
        
        
        tokens = lines[0].split()
        
        if tokens[0] != "MOTION":
            raise ValueError, "Parser Error on MOTION Line"
        
        
        frame_count = self.parse_frame_count()
        secs_per_frame = self.parse_frame_time()
        
        playback_info = PlaybackInfo(frame_count, secs_per_frame * 1000)
        
        
        motionlines = self.lines[3:3+frame_count]
            
        channel_lists = self.parse_channel_lists(motionlines)

        return MotionInfo(channel_lists, playback_info)
    
    
    def check_valid_motion_section(self):
        tokens = self.lines[0].split()
        
        if tokens[0] != "MOTION":
            raise ValueError, "Parser Error on MOTION Line"
    
    def parse_frame_count(self):
        tokens = self.lines[1].split()
        return int(tokens[1])

    def parse_frame_time(self):
        tokens = self.lines[2].split()
        return float(tokens[2])
    
    def parse_channel_lists(self, motion_lines):
        
        
        firstlinetokens = motion_lines[0].split()
        channel_count = len(firstlinetokens)
        
        
        channels = []
        
        for i in range(channel_count):
            channels.append([])
        
        for line in motion_lines:
            tokens = line.split()
            
            for column_number, token in enumerate(tokens):
                
                #converting to a float
                token_float = float(token)
                
                channels[column_number].append(token_float)
    
        return channels




class MotionInfo():
    def __init__(self, channel_lists, playback_info):
        self.channel_lists = channel_lists
        self.next_channel = 0
        
        
        self.playback_info = playback_info
        
    def get_playback_info(self):
        return self.playback_info
    
    PlaybackInfo = property(get_playback_info)

    def get_playback_info(self):
        return self.playback_info
    
    def get_channel(self, number):
        return self.channel_lists[number]
    
    def get_next_channel(self):
        channel = self.channel_lists[self.next_channel]
        self.next_channel += 1
        return channel
        
        









class PlaybackInfo():
    def __init__(self, frame_count, milliseconds_per_frame):
        self.frame_count = frame_count
        self.milliseconds_per_frame = milliseconds_per_frame
        
        
    def get_frame_count(self):
        return self.frame_count
    
    FrameCount = property(get_frame_count)


    def get_milliseconds(self):
        return self.milliseconds_per_frame
    
    FrameRateInMilliseconds = property(get_milliseconds)










    
    
    
    
    
    




class BvhParser(object):
    def __init__(self):
        
        self.lines = None
        #self.channel_List = []


    def parse(self, filename):
        self.file_name = filename
        self.open_file(filename)
        #self.parse_hierarchy()
        return self.parse_file()
    
    
    
    def parse_file(self):
        
        for current_line_num, line in enumerate(self.lines):
            tokens = line.split()
            if tokens[0] == "HIERARCHY":
                hierarchy_begin = current_line_num + 1
                
            elif tokens[0] == "MOTION":
                hierarchy_end = current_line_num
                motion_begin = current_line_num
                break;
            
            
            
        hierarchy_parser = HierarchyParser()
        motion_parser = MotionParser()
        
        hierarchy_lines = self.lines[hierarchy_begin:hierarchy_end]
        motion_lines = self.lines[motion_begin:]
        
        hierarchyroot = hierarchy_parser.parse(hierarchy_lines)
        
        motion = motion_parser.parse(motion_lines)
        
        
        hierarchyroot.integrate(motion)
        
        return AnimatedFigure(self.file_name, hierarchyroot, motion.PlaybackInfo)
        
        

    def open_file(self, filename):
        in_file = open("../" + filename, "r")
        self.lines = in_file.readlines()
        in_file.close()

    





def main():
    
    bvh_parser = BvhParser()
    
    bvhjointset = bvh_parser.parse("walk.bvh")
   
    bvhjointset.get_root().print_info()
    


if __name__=="__main__":
    main()