from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *

from calculation import *

import sys

from consts import *




class Graph(object):
    def __init__(self, value_list, joint):
        
        self.frame_number = 0
        
        self.joint = joint
        self.values = value_list
        self.pruned_values = list(value_list)
        self.interpolated_values = list(value_list)
        
        self.error_values = []
        
        for i in range(len(self.values)):
            self.error_values.append(0.0)
        
        
        
        maxpoint = max(self.values)
        minpoint = min(self.values)
        
        self.absmax = max([abs(maxpoint), abs(minpoint)])
        
        self.pruning_init()
        self.generate_diffs()
        self.calculate_graph_values()
        self.calculate_original_graph_values()
        self.compression_factor = 1.0
        
        
        
    def DisplayValues(self, frame_number):
        
        return self.interpolated_values[frame_number]
        
        
        
        
        
        
    
    def print_info(self):
        
        print self.values
        print self.pruned_values
        print self.interpolated_values
    
    
    
    
    
    def get_points_remaining(self):
        return self.points_to_prune
    
    PointsRemaining = property(get_points_remaining)
    
    def get_total_points(self):
        return len(self.values)
    
    TotalPoints = property(get_total_points)
    
    
    def pruning_init(self):
        
        self.points_to_prune = len(self.values) - 4
    
    
    def calculate_original_graph_values(self):
        
        self.original_graph_values = []
        
        maxpoint = max(self.interpolated_values)
        minpoint = min(self.interpolated_values)
        
        absmax = max([abs(maxpoint), abs(minpoint)])
        
        
        for point in self.interpolated_values:
            
            if absmax == 0:
                graphpoint = 0
            else:
            
                graphpoint = (point / absmax) * 48 + 50
            
            
            self.original_graph_values.append(graphpoint)
    
    
    
    def calculate_graph_values(self):
        
        self.graph_values = []
        
        
        
        
        for point in self.interpolated_values:
            
            if self.absmax == 0:
                graphpoint = 0
            else:
            
                graphpoint = (point / self.absmax) * 48 + 50
            
            
            self.graph_values.append(graphpoint)
        
        
    def get_redundant_point_tuple(self):
        
        
        candidates_list = self.build_redundant_point_tuple_list()
        
        
        
        candidates_list.sort()
       
                
        if len(candidates_list) > 0:
            self.chosen_index = candidates_list[0][1]
            return (candidates_list[0][0], self)
    
        else:
            return None
        
        
        
    def build_redundant_point_tuple_list(self):
        existing_points = []
        
        for index, value in enumerate(self.pruned_values):
            if value != None:
                
                existing_points.append((index, value))
        
        endpoint_error_tuple_list = [] 
        
        for i in range(2, len(existing_points) - 2):
            
            
            prev2 = existing_points[i-2]
            prev = existing_points[i-1]
            curr = existing_points[i]
            next = existing_points[i+1]
            next2 = existing_points[i+2]
        
            points_affected = range(prev[0] + 1, next[0])
            catmull_values = catmull_rom_approx(prev2[0], prev2[1], prev[0], prev[1], range(prev[0] + 1, next[0]), next[0], next[1], next2[0], next2[1])
        
            
        
            endpoint_diff = self.joint.get_total_endpoint_diff(self, points_affected, catmull_values)
    
            if endpoint_diff != None:
                endpoint_error_tuple_list.append((endpoint_diff,curr[0]))
                if abs(endpoint_diff) < 0.00001:
                    
                    break
    
        return endpoint_error_tuple_list
    
    
    
    
    def remove_chosen_point(self):
        self.remove_point(self.chosen_index)
    
    
    
    def remove_point(self,index):
        if self.points_to_prune < 1:
            return
        
        self.points_to_prune -= 1
        
        self.compression_factor = float(self.points_to_prune + 4) / float(len(self.values))
        
        
        
        self.pruned_values[index] = None
        
        x5 = index
        
        x4 = x5
        
        while self.pruned_values[x4] == None:
            x4 -= 1
            
        x3 = x4 - 1
        
        while self.pruned_values[x3] == None:
            x3 -= 1
        
        
        
        x6 = x5
        
        while self.pruned_values[x6] == None:
            x6 += 1
            
        x7 = x6 + 1
        
        while self.pruned_values[x7] == None:
            x7 += 1
                
        
        
        xrange = []
        
        for xmid in range(x4+1, x6):
            xrange.append(xmid)
            
        #print xrange
        
        
        if max(xrange) >= x6:
            print "x6 error"
            sys.exit()
            
        if min(xrange) <= x4:
            print "x4 error"
            sys.exit()
        
        
        
        changes = catmull_rom_approx(x3, self.values[x3], x4, self.values[x4], xrange, x6, self.values[x6], x7, self.values[x7])
        
        for i in range(len(changes)):
            self.interpolated_values[i + x4 + 1] = changes[i]
            
    
        self.calculate_graph_values()
    
    
    
    
    
    
    
    def generate_diffs(self):
        
        diffs = []
        self.diffdiffs = [None, None]
        
        #print "predd",len(self.values)
        
        for i in range(1, len(self.values) - 2):
            
            diff = self.values[i+1] - self.values[i]
            diffs.append(diff)
            
            
        for i in range(len(diffs) - 1):
            diffdiff = diffs[i+1] - diffs[i]
            self.diffdiffs.append(abs(diffdiff))
            
            
            
        self.diffdiffs += [None, None]
        
        self.maxdiffdiffs = max(self.diffdiffs)
        
            
        
    
    
    
    def get_least_useful_index(self):
        
        existing_points = []
        diffs = []
        
        for index, value in enumerate(self.pruned_values):
            if value != None:
                
                existing_points.append((index,value))
                
        
        for i in range(2, len(existing_points) - 2):
            prev = existing_points[i-1]
            curr = existing_points[i]
            next = existing_points[i+1]
            
            back =  (curr[1] - prev[1]) / float(curr[0] - prev[0])
            forward = (next[1] - curr[1]) / float(next[0] - curr[0])
            
            diffs.append((curr[0],abs(forward - back)))
            
        mini = diffs[0][1]
        minimumindex = diffs[0][0]
        
        for i in range(len(diffs)):
            if diffs[i][1] < mini:
                mini = diffs[i][1]
                minimumindex = diffs[i][0]
        
        
        return minimumindex
    
    
    
    
    def get_least_useful_index2(self):
        
        existing_points = []
        diffs = []
        
        for index, value in enumerate(self.pruned_values):
            if value != None:
                
                existing_points.append((index,value))
                
        
        for i in range(2, len(existing_points) - 2):
            prev2 = existing_points[i-2]
            prev = existing_points[i-1]
            curr = existing_points[i]
            next = existing_points[i+1]
            next2 = existing_points[i+2]
            
            
            catmull_values = catmull_rom_approx(prev2[0], prev2[1], prev[0], prev[1], range(prev[0] + 1, next[0]), next[0], next[1], next2[0], next2[1])
            
            error_total = 0
            
            q = prev[0] + 1
            
            for j in range(q, next[0]):
                k = j - q
                error_total += self.values[j] - catmull_values[k]
            
            
            
            
            back =  (curr[1] - prev[1]) / float(curr[0] - prev[0])
            forward = (next[1] - curr[1]) / float(next[0] - curr[0])
            
            diffs.append((curr[0],abs(error_total)))
            
        mini = diffs[0][1]
        minimumindex = diffs[0][0]
        
        for i in range(len(diffs)):
            if diffs[i][1] < mini:
                mini = diffs[i][1]
                minimumindex = diffs[i][0]
        
        
        return minimumindex
            
            
        
    def get_angular_error(self):
        return sum(self.error_values)
        
                
        
        
                
                
                
                
    
    
    
    
    
    
    
    
    
    
    def eliminate_redundant_point(self):
        
        if self.points_to_prune < 1:
            return
        
        self.points_to_prune -= 1
        
        self.compression_factor = float(self.points_to_prune + 4) / float(len(self.values))
        
        
        
                
                
        minimumvalueindex = self.get_least_useful_index2()
                
        self.pruned_values[minimumvalueindex] = None
        
        
        
        
        
        x5 = minimumvalueindex
        
        x4 = x5
        
        while self.pruned_values[x4] == None:
            x4 -= 1
            
        x3 = x4 - 1
        
        while self.pruned_values[x3] == None:
            x3 -= 1
        
        
        
        x6 = x5
        
        while self.pruned_values[x6] == None:
            x6 += 1
            
        x7 = x6 + 1
        
        while self.pruned_values[x7] == None:
            x7 += 1
                
        
        
        xrange = []
        
        for xmid in range(x4+1, x6):
            xrange.append(xmid)
            
        
        if max(xrange) >= x6:
            print "x6 error"
            sys.exit()
            
        if min(xrange) <= x4:
            print "x4 error"
            sys.exit()
        
        
        
        changes = catmull_rom_approx(x3, self.values[x3], x4, self.values[x4], xrange, x6, self.values[x6], x7, self.values[x7])
        
        for i in range(len(changes)):
            self.interpolated_values[i + x4 + 1] = changes[i]
            
    
        self.calculate_graph_values()
        
        
        
        
        
    
    def display(self, x, y):
        self.onOrtho(x, y)
        
        self.draw_graph(x, y)
        
        self.offOrtho()
        
        
        
        
    def onOrtho(self,x,y):
    
        glMatrixMode(GL_PROJECTION);
    
        glPushMatrix();
    
        glLoadIdentity();
    
        gluOrtho2D(0, x, 0, y);
    
        glMatrixMode(GL_MODELVIEW);
    
        glPushMatrix();
    
        glLoadIdentity();



    def offOrtho(self):
        glMatrixMode(GL_PROJECTION);
    
        glPopMatrix();
        glMatrixMode(GL_MODELVIEW);
    
        glPopMatrix();
        
        
        
        
        
        
    def get_frame_number(self):
        return self.frame_number
    
    
    def set_frame_number(self, frame_number):
        self.frame_number = frame_number
        
        
        
        
    Frame = property(get_frame_number, set_frame_number)
        
        
    def draw_graph(self, x, y):
        
        current_frame = self.Frame
        
        glColor3f(1,1,1)
    
        compression_percentage = '%.0f' % float(self.compression_factor * 100)
        cvalstring = '%.2f' % self.interpolated_values[current_frame]
    
        self.add_text(compression_percentage + " %                                                                     " + cvalstring)
        glPointSize(3)
    
        glBegin(GL_LINES)
        glVertex2f(0, y/2)
        glVertex2f(x, y/2)
        glEnd()
    
    
        glBegin(GL_LINES)
        glColor3f(0,1,0)
    
        upperbottom = self.graph_values[current_frame] + 5
    
        if upperbottom < 100:
            glVertex2f(x/2, 100)
            glVertex2f(x/2, upperbottom)
    
        lowertop = self.graph_values[current_frame] - 5
    
        if lowertop > 0:
            glVertex2f(x/2, 0)
            glVertex2f(x/2, lowertop)
    
    
        
    
    
        glEnd()
        
        glColor3f(0,1,0)
    
    
        glBegin(GL_LINES)
    
        for i in range(len(self.graph_values)-1):
            
            if self.pruned_values[i] == None or self.pruned_values[i+1] == None:
                
                position0 = x/2 - 5*(current_frame - i)
                position1 = position0 + 5
            
                glVertex2f(position0, self.graph_values[i])
                glVertex2f(position1, self.graph_values[i+1])
            
        glEnd()
    
    
    
        glColor3f(1,1,1)
    
        glBegin(GL_POINTS)
    
        i = 0
        for index, value in enumerate(self.graph_values):
        
        
            if self.pruned_values[index] != None:
                position = x/2 - 5*(current_frame - i)
        
                glVertex2f(position, value)
       
            i+=1
        
        glEnd()
        
        
        
        
    def add_text_string(self, x, y, spacing, font, displaystring):
  
        x1 = x;
  
        for character in displaystring:
            glRasterPos3f(x1,y,1);
            glutBitmapCharacter(font, ord(character));
            x1 = x1 + glutBitmapWidth(font,ord(character)) + spacing;


  

    def add_text(self, string):
    
        glPushMatrix();
        glLoadIdentity();
        glColor3f(1,1,1)
        self.add_text_string(20,40,0, GLUT_BITMAP_HELVETICA_12, string); 
        glPopMatrix();
        
        
        
        
    def get_current_channel_data_copy(self):
        return list(self.interpolated_values)
        
        
        
        
        
        
        
        
    
    
    
    
    
    
    
    
    

if __name__=="__main__":
    main()
        
        