from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *






from bvhparser import *
from camera import *
from floor import *
from text import *
from notifiers import *
from filenames import *
from eliminatorvisitor import *


class Viewport(object):
    def __init__(self, y = 0, x = 0):
       
        self.x = x
        self.y = y
        
        self.AutoSize = False
        
        
    def display(self, dim_y, offset_y):
        pass
        
        
        
    def get_x(self):
        return self.x_dim
    
    def set_x(self, x):
        self.x_dim = x
    
    
    
    def get_y(self):
        return self.y_dim
        
    
    def set_y(self, y):
        self.y_dim = y
        
        
    x = property(get_x, set_x)
    y = property(get_y, set_y)
    
    
    def get_autosize(self):
        return self.autosize
    
    def set_autosize(self, boolean):
        self.autosize = boolean
    
    AutoSize = property(get_autosize, set_autosize)
    
    
    
class FigureViewport(Viewport):
    def __init__(self, figure, x = 0, y = 0):
        
        Viewport.__init__(self, x, y)
        
        self.AutoSize = True
        self.DisplayInfo = True
            
            
        self.change_figure(figure)
        
        Keyboard.attach_key_listener(' ', self.toggle_animation)
        Keyboard.attach_key_listener("\t", self.next_joint)
        Keyboard.attach_key_listener("q", self.previous_joint)
        Keyboard.attach_key_listener("x", self.eliminate)
        Keyboard.attach_key_listener("m", self.prepare_approximated_files)
        
        
        
        
        
    def change_figure(self, figure):
       
        self.figure = figure
        
        self.joint_notifier = JointChangeNotifier()
        self.frame_notifier = FrameChangeNotifier()
        
        self.lookatheight = self.figure.LookAtHeight
        
        self.camera = Camera()
        self.floor = Floor()
        self.camera.setrange(40)
        
        self.current_frame = 0
        self.paused = True
        self.animation_speed = self.figure.FrameRateInMillseconds
        
        
        
        
    def get_dispinfo(self):
        return self.display_info
    
    def set_dispinfo(self, boolean):
        self.display_info = boolean
        
    DisplayInfo = property(get_dispinfo, set_dispinfo)
        
        
        
        
        
        
        
    def hookup_frame_observer(self, observer):
        
        self.frame_notifier.register(observer)
        self.frame_notifier.update(self.CurrentFrame)
        
        
        
        
        
        
    def hookup_joint_observer(self, observer):
        
        self.joint_notifier.register(observer)
        self.joint_notifier.update(self.figure.CurrentJoint)
    
    
    
    
    
    
        
        
        
        
    def get_cur_frame(self):
        return self.current_frame
    
    
    def set_cur_frame(self, num):
        self.current_frame = num
        
        
    CurrentFrame = property(get_cur_frame, set_cur_frame)
        
        
    
    def next_frame(self):
        self.current_frame += 1
        
        if self.current_frame >= self.figure.playbackinfo.FrameCount:
            self.current_frame = 0
            
        self.frame_notifier.notify_current_frame_changed(self.CurrentFrame)
    
    def previous_frame(self):
        self.current_frame -= 1
        
        if self.current_frame < 0:
            self.current_frame = self.figure.playbackinfo.FrameCount - 1
            
        self.frame_notifier.notify_current_frame_changed(self.CurrentFrame)
    
    
    def animation_timer(self, value):
        
        
        if not self.paused:
            self.next_frame()
            glutTimerFunc(self.animation_speed, self.animation_timer, 1)
    
    
    
    def start_animation(self):
        self.paused = False
        glutTimerFunc(self.animation_speed, self.animation_timer, 1)
    
    
    
    
    def stop_animation(self):
        self.paused = True
        glutTimerFunc(self.animation_speed, self.animation_timer, 1)
    
    
    
    
    def toggle_animation(self):
        if self.paused:
            self.start_animation()
        else:
            self.stop_animation()
    
    def prepare_approximated_files(self):
        
        #self.figure.write_to_file(self.figure.FileName, "walk3.bvh")
        #return
        visitor = EliminatorVisitor()
        
        visitor.prepare_approximated_files(self.figure)
        
        
            
        
    def eliminate(self):
        visitor = EliminatorVisitor()
        visitor.eliminate(self.figure)
    
    
            

            
    def next_joint(self):
        
        self.figure.next_joint()
        self.joint_change_notify()
        
        
    def previous_joint(self):
        
        self.figure.previous_joint()
        self.joint_change_notify()
    
        
        
    def joint_change_notify(self):
        self.joint_notifier.notify_current_joint_changed(self.figure.CurrentJoint)
        
    
    
    
    def register_joint_observer(self, observer):
        self.joint_notifier.register(observer)
        
        
    def register_frame_observer(self, observer):
        self.frame_notifier.register(observer)
        
        
            
        
    def display(self, dim_y, offset_y):
        
        glViewport (0, dim_y - offset_y - self.y, self.x, self.y)
        
        #setting up camera projections
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        
        
        gluPerspective(45, self.x/self.y, 1, 1000)
        
        #setting up scene
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        
        #getting camera pointed in the right direction
        lookatpoint = self.figure.get_lookat_point(self.current_frame)
        #gluLookAt(lookatpoint[0] + self.camera.getx(), lookatpoint[1] + self.camera.gety(), lookatpoint[2] + self.camera.getz(), lookatpoint[0], lookatpoint[1], lookatpoint[2], 0,1,0)
        gluLookAt(lookatpoint[0] + self.camera.getx(), self.lookatheight + self.camera.gety(), lookatpoint[2] + self.camera.getz(), lookatpoint[0], self.lookatheight, lookatpoint[2], 0,1,0)

        #drawing objects in scene
        self.floor.display()
        self.figure.display(self.current_frame)
        
        if self.DisplayInfo:
            
            Text.add_text(self.figure.CurrentJoint.Name)
            Text.add_text(self.figure.CompressionStringPretty, 80)
        
    
        
        
        
        

#implements JointChangeObserver interface
class GraphCollectionViewport(Viewport):
    def __init__(self, x = 0, y = 0):
        
        self.graph_viewports = []
        
        Viewport.__init__(self, x, y)
        
        
        self.x = x
        self.y = y
        
        

    def add_graph_viewports(self, gvs):
        
        
        
        self.graph_viewports += gvs
        
        
        self.y = 0
        
        for gv in self.graph_viewports:
            
            self.y += gv.y
        
        


    def display(self, dim_y, offset_y):
        
        for g in self.graph_viewports:
            g.display(dim_y, offset_y)
            offset_y += g.y
            
    def update_joint(self, joint):
        
        for i in range(len(self.graph_viewports)):
            self.graph_viewports[i].Graph = joint.graphs[i]
            
    def set_x(self, x):
        self.x_dim = x
        
        for gv in self.graph_viewports:
            
            gv.x = x
            
    def get_x(self):
        return self.x_dim
            
    #Redefined fully because Python doesn't subclass properties properly! Rather a bad language flaw.
    x = property(get_x, set_x)
            
    

    def update_frame(self, frame):
        for i in range(len(self.graph_viewports)):
            self.graph_viewports[i].Frame = frame

    


class GraphViewport(Viewport):
    def __init__(self, x = 0, y = 100):
        
        Viewport.__init__(self, x, y)
        self.x = x
        self.y = y
        
        self.graph = None
        
        
    
    def set_graph(self,graph):   
        self.graph = graph
        
    def get_graph(self):
        return self.graph
    
    
    
    Graph = property(get_graph, set_graph)
    
    
    
    def display(self, dim_y, offset_y):
        
        glViewport (0, dim_y - offset_y - self.y, self.x, self.y)
        
        self.Graph.display(self.x, self.y)
        
        
    def set_joint(self,joint):   
        self.joint = joint
        
        self.Graph.Joint = joint
        
        
        
    def get_joint(self):
        return self.joint
        
        
    Joint = property(get_joint, set_joint)
        
        
    def set_frame(self,frame):   
        self.frame = frame
        
        self.Graph.Frame = frame
        
        
        
    def get_frame(self):
        return self.Graph.Frame
        
        
    Frame = property(get_frame, set_frame)
        
        
        
class QuizViewport(Viewport):
    def __init__(self, x = 0, y = 50):
        
        
        
        Viewport.__init__(self, x, y)
        self.x = x
        self.y = y
        self.text = ""
        
        
    def display(self, dim_y, offset_y):
        
        glViewport (0, dim_y - offset_y - self.y, self.x, self.y)
        
        Text.add_text(self.text)
        
        
    def set_text(self,text):   
        self.text = text
        
        
    def get_text(self):
        return self.text
        
        
    Text = property(get_text, set_text)
        